'use strict';

exports.__esModule = true;
exports.ValidationLayer = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _utils = require('./utils');

var utils = _interopRequireWildcard(_utils);

var _Constant = require('./enums/Constant');

var _Constant2 = _interopRequireDefault(_Constant);

var _FeedbackStrategy = require('./enums/FeedbackStrategy');

var _FeedbackStrategy2 = _interopRequireDefault(_FeedbackStrategy);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ValidationLayer = exports.ValidationLayer = function (_React$Component) {
  _inherits(ValidationLayer, _React$Component);

  function ValidationLayer() {
    _classCallCheck(this, ValidationLayer);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var _this = _possibleConstructorReturn(this, _React$Component.call.apply(_React$Component, [this].concat(args)));

    _this.resetComponent = function () {
      _this.isMultiEntities = _this.props.isMultiEntities;

      _this.touchedFields = {};
      _this.bluredFields = {};
      _this.emittedFields = {};

      _this.isSubmitting = false;
      _this.formWasSubmitted = false;
    };

    _this.setTouchedField = function (fieldId) {
      if (!_this.touchedFields[fieldId]) {
        _this.touchedFields[fieldId] = true;
      }
    };

    _this.setBluredField = function (fieldId) {
      if (!_this.bluredFields[fieldId]) {
        _this.bluredFields[fieldId] = true;
      }
      _this.setTouchedField(fieldId);
    };

    _this.setEmittedField = function (fieldId) {
      if (!_this.emittedFields[fieldId]) {
        _this.emittedFields[fieldId] = true;
      }
    };

    _this.getFormFieldsData = function (props) {
      var formFields = {};

      for (var _iterator = props.fields, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
        var _ref;

        if (_isArray) {
          if (_i >= _iterator.length) break;
          _ref = _iterator[_i++];
        } else {
          _i = _iterator.next();
          if (_i.done) break;
          _ref = _i.value;
        }

        var field = _ref;

        var _utils$getFieldIdPart = utils.getFieldIdParts(props, field),
            dataKey = _utils$getFieldIdPart.dataKey,
            entity = _utils$getFieldIdPart.entity,
            entityId = _utils$getFieldIdPart.entityId,
            attr = _utils$getFieldIdPart.attr;

        if (_this.isMultiEntities && !entityId) {
          throw new Error('`isMultiEntities` prop is set to `true`, but entity has no `id`');
        }

        var fieldId = utils.buildFieldId(dataKey, entityId, attr);
        var fieldStateId = utils.buildFieldDataStateId(fieldId);

        formFields[fieldStateId] = {};

        formFields[fieldStateId][_Constant2.default.FIELD_ID_DATA_ATTRIBUTE] = fieldId;

        var omitDomId = utils.isDefined(field.omitDomId) ? field.omitDomId : props.omitDomId;

        if (!omitDomId) {
          var fieldDomId = utils.buildFieldDomId(dataKey, entityId, attr);
          formFields[fieldStateId].id = fieldDomId;
        }

        var fieldValue = utils.fetchProp(entity, attr);
        var domFieldValue = utils.isDefined(fieldValue) && !utils.isNull(fieldValue) ? fieldValue : '';

        var transformBeforeRender = field.transformBeforeRender || props.transformBeforeRender;

        formFields[fieldStateId].value = transformBeforeRender ? transformBeforeRender(domFieldValue) : domFieldValue;

        formFields[fieldStateId].disabled = _this.isSubmitting || field.disabled;

        var formHandlers = {};

        var _props$handlers = props.handlers,
            onSubmit = _props$handlers.onSubmit,
            sharedHandlers = _objectWithoutProperties(_props$handlers, ['onSubmit']); // eslint-disable-line no-unused-vars

        var omitOnChange = utils.isDefined(field.omitOnChange) ? field.omitOnChange : props.omitOnChange;

        if (!omitOnChange) {
          formHandlers.onChange = _this.handleDomChange;
        }

        var omitOnBlur = utils.isDefined(field.omitOnBlur) ? field.omitOnBlur : props.omitOnBlur;

        if (!omitOnBlur) {
          formHandlers.onBlur = _this.handleDomBlur;
        }

        Object.assign(formFields[fieldStateId], sharedHandlers, field.handlers, formHandlers);
      }

      return formFields;
    };

    _this.handleDomChange = function (event) {
      var domData = utils.getDataFromDom(event);
      _this.handleChange(domData, event);
    };

    _this.handleCustomChange = function (fieldDomId, value) {
      var parsedFieldDomId = utils.parseFieldId(fieldDomId);
      var domData = _extends({}, parsedFieldDomId, { value: value });
      _this.handleChange(domData);
    };

    _this.handleChange = function (domData, event) {
      _this.setTouchedField(domData.fieldId);

      var props = _this.props;


      var field = utils.getField(props, domData);
      var filter = field.filter || props.filter;

      if (filter // if filter function provided
      && domData.value // & there's a value (we don't want to filter empty string!)
      && !filter(domData.value, props) // & filter function returned false
      ) {
          return false; // then ignoring this update
        }

      var transformBeforeStore = field.transformBeforeStore || props.transformBeforeStore;

      if (transformBeforeStore) {
        domData.value = transformBeforeStore(domData.value);
      }

      if (event) {
        var handleChange = utils.getHandler(field, props, 'onChange');
        handleChange(domData, {
          originalEvent: event,
          handleSubmit: _this.handleSubmit
        });
      }

      return _this.handleFieldValidation(field, domData, event);
    };

    _this.handleDomBlur = function (event) {
      var domData = utils.getDataFromDom(event);
      _this.handleBlur(domData, event);
    };

    _this.handleCustomBlur = function (fieldDomId, value, event) {
      var parsedFieldDomId = utils.parseFieldId(fieldDomId);
      var domData = _extends({}, parsedFieldDomId, { value: value });
      _this.handleBlur(domData, event, true);
    };

    _this.handleBlur = function (domData, event) {
      var isCustom = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      _this.setTouchedField(domData.fieldId);

      var props = _this.props;


      var field = utils.getField(props, domData);
      var handleBlur = utils.getHandler(field, props, 'onBlur');

      var transformBeforeStore = field.transformBeforeStore || props.transformBeforeStore;

      if (transformBeforeStore) {
        domData.value = transformBeforeStore(domData.value);
      }

      if (!isCustom && handleBlur) handleBlur(domData, event);

      var feedbackStrategy = utils.getStrategy(_this, field);

      if (feedbackStrategy === _FeedbackStrategy2.default.INSTANT_TOUCHED_ONLY || feedbackStrategy === _FeedbackStrategy2.default.ON_FIRST_BLUR || feedbackStrategy === _FeedbackStrategy2.default.ON_BLUR_ONLY || feedbackStrategy === _FeedbackStrategy2.default.ON_SUCCESS_OR_FIRST_BLUR) {
        _this.handleFieldValidation(field, domData, event);
      }
    };

    _this.validateField = function (field, domData, event) {
      return utils.applyStrategy(_this, field, domData, event);
    };

    _this.handleFieldValidation = function (field, domData, event) {
      var nextState = _this.validateField(field, domData, event);
      _this.setState(nextState);
    };

    _this.handleInstantFieldsValidation = function (props) {
      var context = _this;
      var fields = props.fields;


      var instantFields = fields.filter(function (field) {
        var strategy = utils.getStrategy(context, field);
        if (strategy === _FeedbackStrategy2.default.INSTANT) return true;

        if (strategy === _FeedbackStrategy2.default.INSTANT_TOUCHED_ONLY) {
          var _utils$getFieldIdPart2 = utils.getFieldIdParts(props, field),
              dataKey = _utils$getFieldIdPart2.dataKey,
              entityId = _utils$getFieldIdPart2.entityId,
              attr = _utils$getFieldIdPart2.attr;

          var fieldId = utils.buildFieldId(dataKey, entityId, attr);
          if (context.touchedFields[fieldId]) return true;
        }

        return false;
      });

      var _this$handleBatchVali = _this.handleBatchValidation(props, instantFields),
          validationState = _this$handleBatchVali.validationState;

      return validationState;
    };

    _this.handleBatchValidation = function (props, fields) {
      var validatableFields = fields || props.fields;

      var validationState = {};
      var isInvalid = false;

      for (var _iterator2 = validatableFields, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
        var _ref2;

        if (_isArray2) {
          if (_i2 >= _iterator2.length) break;
          _ref2 = _iterator2[_i2++];
        } else {
          _i2 = _iterator2.next();
          if (_i2.done) break;
          _ref2 = _i2.value;
        }

        var field = _ref2;

        var _utils$getFieldIdPart3 = utils.getFieldIdParts(props, field),
            dataKey = _utils$getFieldIdPart3.dataKey,
            entity = _utils$getFieldIdPart3.entity,
            entityId = _utils$getFieldIdPart3.entityId,
            attr = _utils$getFieldIdPart3.attr;

        var statuses = utils.getDefaultStatuses(props);

        var fieldId = utils.buildFieldId(dataKey, entityId, attr);
        var fieldStateId = utils.buildFieldValidationStateId(fieldId);

        var fieldValue = utils.fetchProp(entity, attr);

        var fieldValidationState = utils.normalizeValidationResults(field.validate, fieldValue, props);

        if (!fieldValidationState.valid) {
          fieldValidationState.status = fieldValidationState.status || statuses.error;
          if (!isInvalid) isInvalid = true;
        } else if (fieldValue) {
          fieldValidationState.status = fieldValidationState.status || statuses.success;
        }

        validationState[fieldStateId] = fieldValidationState;
      }

      return { validationState: validationState, isInvalid: isInvalid };
    };

    _this.handleSubmit = function (event) {
      _this.isSubmitting = true;

      if (!_this.formWasSubmitted) {
        _this.formWasSubmitted = true;
      }

      if (event && event.preventDefault && !event.defaultPrevented) {
        event.preventDefault();
      }

      var nextFormFieldsData = _this.getFormFieldsData(_this.props);

      _this.setState(nextFormFieldsData, _this.handleFormValidation);
    };

    _this.handleFormValidation = function () {
      var _this$handleBatchVali2 = _this.handleBatchValidation(_this.props),
          validationState = _this$handleBatchVali2.validationState,
          isInvalid = _this$handleBatchVali2.isInvalid;

      if (isInvalid) {
        _this.handleFailedFormValidation(validationState);
      } else {
        _this.performSubmit();
      }
    };

    _this.handleFailedFormValidation = function (validationState) {
      _this.setState(validationState, _this.handleFailurePostSubmitAction);
    };

    _this.performSubmit = function () {
      _this.props.handlers.onSubmit({
        onSuccess: _this.handleSuccessPostSubmitAction,
        onFailure: _this.handleFailurePostSubmitAction
      });
    };

    _this.handleSuccessPostSubmitAction = function () {
      _this.resetComponent();

      var nextFormFieldsData = _this.getFormFieldsData(_this.props);
      var nextVaidationState = _this.handleInstantFieldsValidation(_this.props);

      var resetedState = utils.resetState(_this.state, nextFormFieldsData, nextVaidationState);

      _this.setState(resetedState);
    };

    _this.handleFailurePostSubmitAction = function () {
      _this.isSubmitting = false;

      var nextFormFieldsData = _this.getFormFieldsData(_this.props);

      _this.setState(nextFormFieldsData);
    };

    _this.printDeprecationWarning = function () {
      if (_this.warned) return;

      // eslint-disable-next-line no-console
      console.warn('[Validation Layer Warning]: Direct rendering of the children is deprecated. ' + 'Use children-as-function way to render your form.');
      _this.warned = true;
    };

    _this.resetComponent();
    return _this;
  }

  ValidationLayer.prototype.componentWillMount = function componentWillMount() {
    var nextFormFieldsData = this.getFormFieldsData(this.props);
    var nextVaidationState = this.handleInstantFieldsValidation(this.props);
    var nextState = _extends({}, nextFormFieldsData, nextVaidationState);

    this.setState(nextState);
  };

  ValidationLayer.prototype.componentDidMount = function componentDidMount() {
    var setValidationLayerStateResetCallback = this.props.setValidationLayerStateResetCallback;


    if (setValidationLayerStateResetCallback) {
      setValidationLayerStateResetCallback(this.handleSuccessPostSubmitAction);
    }
  };

  ValidationLayer.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var nextFormFieldsData = this.getFormFieldsData(nextProps);
    var nextVaidationState = this.handleInstantFieldsValidation(nextProps);
    var nextState = _extends({}, nextFormFieldsData, nextVaidationState);

    this.setState(nextState);
  };

  ValidationLayer.prototype.render = function render() {
    var children = this.props.children;

    var layer = utils.createLayer(this);

    if (typeof children === 'function') {
      return children(layer);
    }

    this.printDeprecationWarning();
    return _react2.default.cloneElement(children, { form: layer });
  };

  return ValidationLayer;
}(_react2.default.Component);

ValidationLayer.propTypes = {
  children: _react.PropTypes.oneOfType([_react.PropTypes.element.isRequired, _react.PropTypes.func.isRequired]),

  dataKey: _react.PropTypes.string,
  dataType: _react.PropTypes.oneOf([_Constant2.default.COLLECTION_DATA_TYPE]),

  handlers: _react.PropTypes.shape({
    onSubmit: _react.PropTypes.func.isRequired
  }).isRequired,

  feedbackStrategy: _react.PropTypes.oneOf([_FeedbackStrategy2.default.INSTANT, _FeedbackStrategy2.default.INSTANT_TOUCHED_ONLY, _FeedbackStrategy2.default.ON_CHANGE, _FeedbackStrategy2.default.ON_BLUR_ONLY, _FeedbackStrategy2.default.ON_FIRST_BLUR, _FeedbackStrategy2.default.ON_SUCCESS, _FeedbackStrategy2.default.ON_SUCCESS_OR_FIRST_BLUR, _FeedbackStrategy2.default.ON_SUBMIT]),

  fields: _react.PropTypes.arrayOf(_react.PropTypes.shape({
    dataKey: _react.PropTypes.string, // Can be global
    dataType: _react.PropTypes.oneOf([// Can be global
    _Constant2.default.COLLECTION_DATA_TYPE]),
    attr: _react.PropTypes.string.isRequired,
    validate: _react.PropTypes.func,
    validateAsync: _react.PropTypes.func,
    handlers: _react.PropTypes.object, // Can be global
    filter: _react.PropTypes.func, // Can be global
    transformBeforeStore: _react.PropTypes.func, // Can be global
    transformBeforeRender: _react.PropTypes.func, // Can be global
    omitDomId: _react.PropTypes.bool, // Can be global
    omitRef: _react.PropTypes.bool, // Can be global
    omitOnChange: _react.PropTypes.bool, // Can be global
    omitOnBlur: _react.PropTypes.bool, // Can be global

    feedbackStrategy: _react.PropTypes.oneOf([// Can be global
    _FeedbackStrategy2.default.INSTANT, _FeedbackStrategy2.default.INSTANT_TOUCHED_ONLY, _FeedbackStrategy2.default.ON_CHANGE, _FeedbackStrategy2.default.ON_BLUR_ONLY, _FeedbackStrategy2.default.ON_FIRST_BLUR, _FeedbackStrategy2.default.ON_SUCCESS, _FeedbackStrategy2.default.ON_SUCCESS_OR_FIRST_BLUR, _FeedbackStrategy2.default.ON_SUBMIT])
  }).isRequired).isRequired,

  isMultiEntities: _react.PropTypes.bool,

  successStatus: _react.PropTypes.string,
  errorStatus: _react.PropTypes.string,

  setValidationLayerStateResetCallback: _react.PropTypes.func
};